#!/bin/bash

# Description:
#
# Building your own libpst before exporting your contacts and calendars
# from MS-Outlook is essential because Ubuntu (and possibly other Linux
# distributions) do NOT update libpst quite often.
#
# The libpst version of Ubuntu 04.11 is v0.6.41 and still contains errors that
# are already fixed in the new version that we try to build
# by executing this script, e. g.
#
#     LibPST 0.6.45 (2009-11-18)
#     ===============================
#     * patch from Hugo DesRosiers to export categories and notes into vcards.
#     * extend that patch to export categories into vcalendar appointments also.

# !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
# Without the new version (containing this patch) you will loose
# - the notes in your contacts and
# - the categories in the contacts and calendar events!
# !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

# --- LEGAL INFORMATION --------------------------------------------------------
# THE AUTHOR OF THIS SCRIPT HEREBY GIVES UP ALL RIGHTS ALLOWING YOU TO DO
# WHAT YOU WANT (E. G. COMMERCIAL AND NON-COMMERCIAL USE OF ORIGINAL OR MODIFIED
# SCRIPTS WITHOUT REFERING TO THE ORIGINAL SOURCE).
# THE AUTHOR OF THIS SCRIPT WANTS TO STAY ANONYMOUS BUT CAN BE CONTACTED
# VIA THE USER NAME "DANUBIO" AT THE WIKI SITE "wiki.ubuntuusers.de" WHERE
# THE ORIGINAL VERSION OF THIS SCRIPT HAS BEEN PUBLISHED FIRST UNDER THE
# (WIKI PAGE) NAME "OUTLOOK2EVOLUTION".
# ------------------------------------------------------------------------------

# #######################################################################
# Change log (date format is mm/dd/yyyy):
# 01/01/2011 Initial version
# 08/09/2011 * Refactoring: LibPST version can now be set using a variable
#            * Uses LibPST 0.6.53 (as of 07/11/2011) now instead of 0.6.49
# #######################################################################


# For more information about libpst (containing readpst) see
# http://www.five-ten-sg.com/libpst/

clear

echo "Starting the build build process for 'readpst'..."

echo "Starting..." > 2_build_libpst.log


# Variable that defines the libPST version (and therefore the file name)
# to download and install.
# For a list of available versions of libpost see:
# http://www.five-ten-sg.com/libpst/packages/
libpstversion="libpst-0.6.53"


# Download a stable version of libpst (that is known to work).
# In case of problems (compilation errors, required bug fixes in libpst etc.)
# try to specify a newer version by looking at the version list
# http://www.five-ten-sg.com/libpst/packages/
if [ -f "$libpstversion.tar.gz" ]
then
  echo "Skipped downloading the file since it already exists in the current folder..."
else

  echo "Downloading the source code of 'libpst'..."
  wget --no-host-directories --no-directories "http://www.five-ten-sg.com/libpst/packages/$libpstversion.tar.gz" >> 2_build_libpst.log

  # Decompress download archive file
  echo "Decompressing the downloaded archive file..."
  rm -f -r "$libpstversion" >> 2_build_libpst.log
  tar -xvzf "$libpstversion.tar.gz" >> 2_build_libpst.log

fi



# Check point required to continue...
if [ ! -d "$libpstversion" ]
then

  echo "ERROR: The source code could not be downloaded or extracted..."
  exit -1

fi



cd "$libpstversion"



# Clean up old compilation results (just to be sure a full recompile is done)
echo "Cleaning up old compilation results..."
make clean >> ../2_build_libpst.log



# Prepare compilation process
echo "Preparing compilation..."
if [ ! -f Makefile ]   # WARNING: A corrupted or wrongly configured Makefile will be reused!
then
  echo "Creating Makefile now... (can take a few minutes)"
  ./configure --disable-python >> ../2_build_libpst.log
else
  echo "Reuse existing Makefile..."
fi



# Compile
echo "Compiling now... (can take a few minutes)"
make >> ../2_build_libpst.log



# Perform self tests
echo "Performing self tests on the compiled file..."
make check >> ../2_build_libpst.log


# Delete existing version to recognize compilation failure...
rm -f ../readpost


cp src/readpst .. >> ../2_build_libpst.log
cd ..
chmod +x readpst >> 2_build_libpst.log



echo
echo "Trying to find out the version number of readpost..."
echo



# Do a test call to show the version information
echo "==== Version information about readpst: ======"
./readpst -V
echo "=============================================="
echo



# Final test for success or failure
if [ -f readpst ]
then
  echo "Build was successful!"
else
  echo "Build failed!"
  echo "Please see the log file '2_build_libpst.log' to find out the reason..."
  echo
  echo "Please make sure to have installed the required development packages"
  echo "by executing the script '1_setup_packages.sh' first!"
  echo
  exit -2
fi

echo

# Signal success
exit 0









