#!/bin/bash

# Description:
#
# Converts mbox files in all sub folders into an evolution conform file and folder structure
#
# Wrongly encoded folder names will be converted correctly (required for non US character encodings of outlook).
#
# Call once to fix all wongly encoded file names (caused by readpst)
# starting by the current directory and processing all sub folders.
#
# Invariants:
# - Does not change the source folder structure, file or folder names!

# For a documentation of the mbox format and the evolution specific folder structure
# and supplementary files see:
# http://live.gnome.org/Evolution/FAQ#How_can_I_use_local_folders_of_Evolution_with_other_Mail_User_Agents_.28MUAs.29
# http://tools.ietf.org/html/rfc4155
# http://qmail.org./man/man5/mbox.html
#
# Brief description of the mbox file and folder structure:
# - The local emails (sub folders under the node "On this computer")
#   are stored in ~/.evolution/mail/local/
# - For each mail folder "folder_i" the following files and folders may exist:
#   - folder_i (optional file containing all emails of this folder using the mbox format)
#   - folder_i.sbd (folder containing all sub folders of this mail folder)
#   - folder_i.cmeta (optional file)
#   - folder_i.ibex.index (optional file)
#   - folder_i.ibex.index.data (optional file)
#   - folder_i.ev-summary (optional file)
#   - folder_i.ev.summary-meta (optional file)
#
# Evolution will recreate the optional index files when you restart evolution
# so it suffice to copy the mbox files using the correct file name
# and creating the correct folder structure!


# --- LEGAL INFORMATION --------------------------------------------------------
# THE AUTHOR OF THIS SCRIPT HEREBY GIVES UP ALL RIGHTS ALLOWING YOU TO DO
# WHAT YOU WANT (E. G. COMMERCIAL AND NON-COMMERCIAL USE OF ORIGINAL OR MODIFIED
# SCRIPTS WITHOUT REFERING TO THE ORIGINAL SOURCE).
# THE AUTHOR OF THIS SCRIPT WANTS TO STAY ANONYMOUS BUT CAN BE CONTACTED
# VIA THE USER NAME "DANUBIO" AT THE WIKI SITE "wiki.ubuntuusers.de" WHERE
# THE ORIGINAL VERSION OF THIS SCRIPT HAS BEEN PUBLISHED FIRST UNDER THE
# (WIKI PAGE) NAME "OUTLOOK2EVOLUTION".
# ------------------------------------------------------------------------------


# #######################################################################
# Change log (date format is mm/dd/yyyy):
# 01/01/2011 Initial version
# 08/10/2011 * Added legal information
#            * Separated file name from file content conversion of the character encodings
#            * Fixed problem: Delete the "folders.db" that caused inconsistent caching
#                             (showing duplicated emails when importing the same
#                              outlook file twice using this script)
#            * Fixed problem: Create empty mbox file even if the folder is empty
#                             to avoid an Evolution problem deleting the folder
#            * Fixed bug: Folder names starting with a dot did not work in
#                         Evolution since this means a hidden folder in Linux.
#                         Replacing the dot by an underscore now.
# #######################################################################



# ----------------------------------------------------------------------------
# To convert character encodings of the file/folder names and the file contents
# to Linux/Ubuntu without loosing special characters you have to specify the
# encoding of your Outlook file.
#
# Source endcoding - change this value accordingly when using a non european outlook version
# For a list of valid encodings see (list of encoding values):
# http://www.debian.org/doc/manuals/reference/ch11.en.html#list-of-encoding-values
# 
# File and folder names can be converted independently from the file contents
# since this seems to be required for some Outlook data files for unknown reasons.
FILE_NAME_ENCDN=iso-8859-1
FILE_CONTENT_ENCDN=iso-8859-1
#iso-8859-1
# To switch off the conversion set this variable to "n" (n=no) (y=yes)
# Normally the mbox file need not to be converted anymore since the contained
# characters are already OK but who knows what outlook does...
DO_FILE_CONTENT_ENCDN_CONVERSION=n
# ----------------------------------------------------------------------------



# ----------------------------------------------------------------------------
#  Fixes the encoding of each file and directory name in the current folder.
#
#  Function input params:
#  $1 = current target folder
#  $2 = current source folder
#  $3 = previous target folder ("parent")
# ----------------------------------------------------------------------------
FixFileEncoding()
{

  local newfilename

  local l_curOutFolder=$1
  local l_curFolderName=$2
  local l_curOutParentFolder=$3

  local l_nextOutFolder


  # Process all files in the current folder, THEN recursivly process all sub folders...
  for aFile in $(ls -A1 --group-directories-first | tac)  # Note: The parameter "A one" lists all files and folders without the implicit "." and ".." folders (one per row)
  do
    
    #echo "Found: $aFile"

    if [ $aFile != "." -a $aFile != ".." ]; # just to be sure
    then

      # Work around for german and english encoding problems.
      # File names containing invalid characters are displayed using one of
      # the sub strings used in the sed command sometimes.
      # Therefore we remove this sub string if it occurs...
      newfilename=`echo "$aFile" | sed 's/(ungültige Kodierung)\|(invalid encoding)//gI' | iconv -f $FILE_NAME_ENCDN -t utf-8`

      # Work around: Replace leading dot by underscore since hidden files or
      #              folders do not work as Evolution email folder.
      newfilename=${newfilename/#./_}

      #echo "Changed file name from $aFile  to  $newfilename"


      # Is it a file?
      if [ -f "$aFile" ];
      then

        #  Is the file the mbox file (containing all emails of the current folder)?
        if [ $aFile = "mbox" ];
        then

          # Rename the mbox file and move it one folder up (evolution email structure)
          # The file content is written into a file named like the folder...

          # Disable the following line to disable the mbox file content encoding conversion!
          # It is unclear when this is required. Just check your imported emails after this
          # script run successfuly to check out whether the special character within your emails
          # are displayed wrongly.
          if [ $DO_FILE_CONTENT_ENCDN_CONVERSION = "y" ];
          then
              iconv -f $FILE_CONTENT_ENCDN -t UTF-8 "$aFile" > "$l_curOutParentFolder/$l_curFolderName" 
              # Reminder for work around: iconv -f $FILE_CONTENT_ENCDN -t UTF-8 "$aFile" | cat > "$l_curOutParentFolder/$l_curFolderName"
              # echo "file content conversion applied"
          else
              # Copy the unconverted file - disabled when you enabled the conversion above
              cp "$aFile" "$l_curOutParentFolder/$l_curFolderName"
              # echo "no file content conversion applied"
          fi

        else

          echo "WARNING: File ignored since it is no mbox: $aFile"

        fi
      # It is not a file. Is it a folder?
      elif [ -d "$aFile" ];
      then

        #  Create new target folder for source sub folder and process next sub folder recursively
        l_nextOutFolder="$l_curOutFolder/$newfilename.sbd"
        #echo "Try to mkdir: $l_nextOutFolder"
        mkdir "$l_nextOutFolder"

        # Create empty mbox file if none exists yet
        # (required to allow the deletion of a folder tree in Evolution
        # without the error message "... folder does not exist...")
        touch "$l_curOutFolder/$newfilename"
	# echo "touching $newfilename"

        cd "$aFile"

        FixFileEncoding "$l_nextOutFolder" "$newfilename" "$l_curOutFolder"  # RECURSION!!!

        # Go up one folder in the source folder hierarchy
        cd ..

        #Note: Since the variables are local we do NOT need to reset their
        #      values to "go up" one target folder after processing the sub folder.

      else

        echo "ERROR: Object is neither a file nor folder."
        echo "       The object will be ignored: $aFile"

      fi

    fi
    
  done

}




# ----------------------------------------------------------------------------
# MAIN
# ----------------------------------------------------------------------------

clear

if [ ! -d "outlook_raw" ];
  then

  echo "ERROR: The sub folder 'outlook_raw' does not exist!"
  echo "Please execute the script './3_export_outlook_data.sh' first"
  echo "before calling this script!"

  exit -1
fi

cd "outlook_raw"

echo "Starting the conversion at: $PWD"

# Set the field delimiter chars to "new line" and ??? (why b?)
# This is required to overcome problems with files and folders containing spaces
# etc. within their name (their file name will be split into part instead
# of treating them as on file name because spaces and new lines are filed
# separators by default!).
IFS=$(echo -en "\n\b")

#echo "IFS=$IFS" | od -b  # debug output


# Shutdown evolution (server) to add new mbox files.
# This is required since the new mbox files will only be reindex by
# restarting evolution.
evolution --force-shutdown


# Folder where evolution stores the local emails
evolution_localmail="$HOME/.evolution/mail/local"


# Assure that the evolution mail folder is there where we expect it to be
if [ ! -d "$evolution_localmail" ];
  then
  echo "Folder does not exist:"
  echo "$evolution_localmail"
  echo "Please check whether Evolution is installed correctly!"
  echo "The script will be stopped now!"
  exit -2
fi


# Folder name used as the "root" to import the emails
initial_target_folder="$evolution_localmail/outlook.sbd"


if [ -d "$initial_target_folder" ];
then
  echo
  echo "WARNING !!! --------------------------------------------------------"
  echo "The target folder used as root folder for the import already exists:"
  echo "$initial_target_folder"
  echo "All emails in this folder will be deleted if you continue (by answering 'y')!"
  echo "To import more than one outlook pst file (e. g. all outlook archive files)"
  echo "you have to rename the root target folder 'outlook' in Evolution"
  echo "before importing the next pst file!"
  echo
  echo "Do you really want to delete the existing root email folder in evolution"
  echo "loosing all your emails in there? (y=yes, n=no to abort the script)"

  read answer
  if [ "$answer" != "y" ] && [ "$answer" != "Y" ]
  then
    echo "Aborting the import process by exiting the script..."
    exit -3
  fi

  echo "Deleting existing target folder:"
  echo "$initial_target_folder"
  
  # delete outlook root folder under local mail
  rm -f -r "$initial_target_folder"
  # delete index file in the parent folder to avoid viewing emails from the index that don't exist anymore
  rm -f "$initial_target_folder/../outlook"
  rm -f "$initial_target_folder/../outlook.cmeta"
  rm -f "$initial_target_folder/../outlook.ibex.index"
  rm -f "$initial_target_folder/../outlook.ibex.index.data"
  # Work around: Starting this script  several times (e. g. to test the import)
  # shows duplicated emails sometimes (Ubuntu 10.04) because the internal
  # folder cache of Evolution is out of snyc. So we simply delete it since
  # Evolution rebuilds it at the next restart.
  # Evolution must be shut down before (assured above).
  # Todo: Should we make a backup of the file?
  rm -f "$evolution_localmail/folders.db"
fi


# Create "root" folder for imported email hierarchy
mkdir "$initial_target_folder"


# Initial name of the current target folder
CURFOLDER="outlook"

# Initial path of the last target folder
LASTOUTDIR="$evolution_localmail"


# Call the conversion function (recursion!)
FixFileEncoding "$initial_target_folder" "$CURFOLDER" "$LASTOUTDIR"

# Leave "outlook_raw"
cd ..


# todo: Restart evolution (in background?) - this line is blocking (no good idea)
#evolution

echo "---- Conversion completed! -----"
echo "Your emails can be found in evolution now under the folder:"
echo "local --> outlook"
echo
echo "Please start evolution again now."
echo
echo "NOTES:"
echo "1. Evolution needs to reindex your imported emails so the"
echo "   emails may not show up immediately when you select a folder"
echo "   of the newly imported emails for the first time!"
echo "2. The imported emails will all appear as 'unread'"
echo "   (the answer status cannot be migrated to evolution)."
echo

# Signal success
exit 0


